// PKTUSBAPI_Console_Demo.cpp : main project file.
/*!
** NAME: \file PKTUSBAPI_Console_Demo.cpp									\n
**																			\n
** (C) Copyright PREHKEYTEC 2018.      									    \n
**																			\n
** PURPOSE:																	\n
**  This module contains routines for cpp PKTUSB API demonstration.			\n

**																			\n
** No Warranty																\n
** The Software is being delivered to you "AS IS" and PrehKeyTec GmbH		\n
** makes no warranty as to its use or performance. Details see readme.		\n
**																			\n
** The PKTUSBAPI is written in c# (managed code).							\n
** It is not allowed to use managed code in static variables.				\n
** DevicePath is used as ifdentifier instead of	managed class USB_Device.	\n
**																			\n
**  \brief This console C++ demo works with PKTUSBAPI.			   	        \n
**																		   	\n
** FUNCTIONS:																\n
**  main routine													        \n
**  MyMSR event														        \n
**  MyPOSkey event															\n
**	myAPI_OnDeviceStatusChanged												\n
**																			\n
** \author          B Grossmannn      	\version       1.0				    \n
**																			\n
**   									\date   	   17 Jan 2018			\n
**																			\n
** REVISION HISTORY:														\n
**																			\n
**  DATE       	VERSION	PROGRAMMER     	DESCRIPTION							\n
**  ========== 	======= ============== 	=======================		        \n
**  17. Jan 18    1.0  	Grossmann B.   	Module created 						\n
*/

// INCLUDES

#include "stdafx.h"
#include <string>
#include <vcclr.h>
#include <msclr/marshal.h>
#include <msclr\marshal_cppstd.h>
#include <iostream>
#include <conio.h>
using namespace System;
using namespace PKTUSBAPI;

static std::string  DevicePath = ""; // use the device path instead manaded class USB_Device

/**
 ** static void MyMSR(Object^ sender, EventArguments::OnReceivedMSR^ e)
 ** \brief  The MSR event handler.
 **/
static void MyMSR(Object^ sender, EventArguments::OnReceivedMSR^ e)
{
           //Show received event data 
            std::cout << "\nTrack 1: ";
			System::String^ msr1 = e->MSRtrack1;
			std::string unmanagedmsr1 = msclr::interop::marshal_as<std::string>(msr1); // https://msdn.microsoft.com/en-us/library/bb384865.aspx
			std::cout << unmanagedmsr1;

            std::cout << "\nTrack 2: ";
			System::String^ msr2 = e->MSRtrack2;
			std::string unmanagedmsr2 = msclr::interop::marshal_as<std::string>(msr2);
			std::cout << unmanagedmsr2;

            std::cout << "\nTrack 3: ";
			System::String^ msr3 = e->MSRtrack3;
			std::string unmanagedmsr3 = msclr::interop::marshal_as<std::string>(msr3);
			std::cout << unmanagedmsr3;
			std::cout << "\n";
}

/**
 ** static void MyPOSkey(Object^ sender, EventArguments::OnReceivedPOSkey^ e)
 ** \brief  The POS key event handler.
 **/
static void MyPOSkey(Object^ sender, EventArguments::OnReceivedPOSkey^ e)
{
           //Show received event data 
            std::cout << "\nPOSkey: ";
			System::Int32^ POSkey = e->Key; // get POSkey number 
			std::cout << (int)POSkey; // print it
			if(e->Pressed == true) // check if the key was pressed or released
				std::cout << " pressed.";
			else
				std::cout << " released.";
}
/**
 ** static void myAPI_OnDeviceStatusChanged(Object^ sender, EventArguments::OnDeviceStatusChanged^ e)
 ** \brief  detect if the device is removed
 **/
static void myAPI_OnDeviceStatusChanged(Object^ sender, EventArguments::OnDeviceStatusChanged^ e)
{
    //Do something only if a device was attached or detached
    if (e->ConnectedStatusChanged)
    {
        //Check the usage page of the de-/attached device to see if it is a PrehKeyTec standard device
        if (e->ChangedDevice->UsagePage == 0xfffa) // UsagePage FFFA is a standard programmable PrehKeyTec keyboard
        {
			System::String^ changeddevicepath = e->ChangedDevice->DevicePath; // get the device path of changed device
			std::string unmanagedpath = msclr::interop::marshal_as<std::string>(changeddevicepath); // convert the device path of changed device into unmanaged code
            //Check if myDevice was detached
            if (DevicePath != "") // a device was used
				if(DevicePath == unmanagedpath) // the changed device is the same as the already used device
					if( e->ChangedDevice->IsConnected == false) // the device was removed
            {
                std::cout << "\nDevice closed";
				DevicePath = "";  //Clear static path for main routine 
            }
        }
    }
}

/**
 ** int main(array<System::String ^> ^args)
 ** \brief  The main routine.
 **/ 
int main(array<System::String ^> ^args)
{
	PKTUSBAPI_Class^ myAPI = nullptr; // handle declaration of the API Class
	USB_Device^ myDevice = nullptr;	 // handle declaration of the device

	myAPI = gcnew PKTUSBAPI_Class(); //managed object instantiation
	if (myAPI == nullptr) //managed object instantiation failed?
	{
		std::cout << "No PKTUSB API found!\n";
		return 0; // quit
	}
	//
	//Set up on device status changed event 
    myAPI->OnDeviceStatusChanged += gcnew EventHandler<EventArguments::OnDeviceStatusChanged^>(myAPI_OnDeviceStatusChanged);
	//Show the library versions 
 	char info[] ="\nGet any data from PrehKeyTec keyboard using PosKey, PosMSR, PosKeylock.\nThe PrehKeyTec keyboard need to be programmed in OPOS mode!\n";
	std::cout << info;
	std::cout << "\nPKTUSBAPI Console Demo started...";
    std::cout << "\nPKTUSBAPI.dll Version: ";
	System::String^ API_version = myAPI->VersionAPI; // read managed version of the API
	if(API_version) // version read? 
	{
		std::string unmanagedAPI_version = msclr::interop::marshal_as<std::string>(API_version); // convert managed in unmanaged code
		std::cout << unmanagedAPI_version; // print version
	}
	std::cout << "\nPKTUSBxx.dll Version: ";
	System::String^ DLL_version = myAPI->VersionDLL; // read managed version of the DLL
	if(DLL_version) // version read? 
	{
		std::string unmanagedDLL_version = msclr::interop::marshal_as<std::string>(DLL_version);
		std::cout << unmanagedDLL_version;
	}
	std::cout << "\nInitialize the PKTUSBAPI";
    std::cout << "\n--------------------------------------------------\n\n";
	std::cout << '\n' << "Press a key to quit...";
	std::cout << "\nsearching for devices!";
	do
	{
		if(DevicePath == "")
		{
			myDevice = myAPI->OpenDevice(); //try to open a device
			if (myDevice != nullptr) //sucessfully opened a device
			{
				int i = 10;
				System::String^ MyPath = myDevice->DevicePath; // store path of opened device
				DevicePath = msclr::interop::marshal_as<std::string>(MyPath); // https://msdn.microsoft.com/en-us/library/bb384865.aspx
				std::cout << "\nDevice opened";
				std::cout << "\nDevicePath and version of opened device";
				std::cout << "\n--------------------------------------------------\n";
				System::String^ Device_path = myDevice->DevicePath;
				std::string unmanagedDevice_path = msclr::interop::marshal_as<std::string>(Device_path);
				std::cout << unmanagedDevice_path;	// Display path of connected device
				std::cout << "\n";
				System::String^ Device_version = myDevice->VersionString;
				std::string unmanagedDevice_version = msclr::interop::marshal_as<std::string>(Device_version);
				std::cout << unmanagedDevice_version;	// Display firmware version of connected device		
				myDevice->PlayTone(Constants::TONE_CALL, 20); // Play a sound 20% for startup
				{ // Control LED example
					int acceptColor = Constants::ACCEPT_ORANGE; //Accept LED color: Orange
					int acceptFlash = Constants::ACCEPT_FLASH_600ms;//Accept LED speed: flashing 600ms 
					bool num = false; //Switch Off Num-Lock LED
					bool scroll = true; //Switch On Scroll-Lock LED
					bool caps = true; //Switch On Caps-Lock LED
					//Send command to control the device LEDs
					myDevice->SetLEDs(num, caps, scroll, acceptColor, acceptFlash);
				}		               
				myDevice->OnReceivedMSR += gcnew EventHandler<EventArguments::OnReceivedMSR^>(MyMSR); // add MyMSR handler
				myDevice->OnReceivedPOSkey += gcnew EventHandler<EventArguments::OnReceivedPOSkey^>(MyPOSkey); // add MyPOSkey handler
			}
		}
	}while(_kbhit()==false);
	if(myDevice) // if a device is opened
		myDevice->Close(); //  close PrehKeyTec decive	
	return 0;
}

 